START TRANSACTION;

DELIMITER //

# -------------------------------------------------------------------------------------------
# Label: departments
# -------------------------------------------------------------------------------------------
DROP PROCEDURE IF EXISTS Department_GetIdByName//
CREATE PROCEDURE Department_GetIdByName(IN pDepartmentName varchar(200))
READS SQL DATA
BEGIN
  SELECT departments.ID as ID
  FROM departments
  WHERE departments.Name = pDepartmentName;
END//

DROP PROCEDURE IF EXISTS Department_GetIdByExternalID//
CREATE PROCEDURE Department_GetIdByExternalID(IN pDepartmentExternalID VARCHAR(9))
READS SQL DATA
BEGIN
  SELECT departments.ID as ID
  FROM departments
  WHERE departments.ExternalID = pDepartmentExternalID;
END//

DROP PROCEDURE IF EXISTS Departments_Get//
CREATE PROCEDURE Departments_Get (IN pFacultyID INT)
READS SQL DATA
BEGIN
  IF pFacultyID <=> 0 THEN
    SELECT departments.* FROM departments
    ORDER BY departments.Name ASC;
  ELSE
    SELECT  departments.* -- ID, Name, FacultyID
    FROM departments
    WHERE departments.FacultyID = pFacultyID
    ORDER BY departments.Name ASC;
  END IF;
END //

# create department or return existing
DROP FUNCTION IF EXISTS Department_Create //
CREATE FUNCTION Department_Create (
  pName VARCHAR(200) CHARSET utf8,
  pFacultyID INT(11)
) RETURNS INT(11) # department id or -1 if failed
NO SQL
BEGIN
  DECLARE EXIT HANDLER FOR SQLEXCEPTION RETURN -1;

  INSERT INTO departments (Name, FacultyID) VALUES (pName, pFacultyID)
  ON DUPLICATE KEY UPDATE
    departments.ID = LAST_INSERT_ID(departments.ID);
  RETURN LAST_INSERT_ID();
END //

DROP FUNCTION IF EXISTS Department_Search //
CREATE FUNCTION Department_Search (
  pName VARCHAR(200) CHARSET utf8,
  pFacultyID INT(11)
) RETURNS INT(11) # department id of -1 if not found
READS SQL DATA
  RETURN COALESCE((
                    SELECT departments.ID FROM departments
                    WHERE   departments.Name = pName AND departments.FacultyID = pFacultyID LIMIT 1
                  ), -1)//

# -------------------------------------------------------------------------------------------
# Label: job positions
# -------------------------------------------------------------------------------------------
DROP PROCEDURE IF EXISTS JobPositions_Get//
CREATE PROCEDURE JobPositions_Get ()
READS SQL DATA
  SELECT  job_positions.* -- ID, Name
  FROM    job_positions
  ORDER BY job_positions.Name //

DROP PROCEDURE IF EXISTS JobPosition_GetIdByName//
CREATE PROCEDURE JobPosition_GetIdByName(IN pJobPositionName varchar(200))
READS SQL DATA
BEGIN
  SELECT job_positions.ID as ID
  FROM job_positions
  WHERE job_positions.Name = pJobPositionName;
END//

DROP FUNCTION IF EXISTS JobPosition_Create //
CREATE FUNCTION JobPosition_Create (
  pJobPositionName varchar(200) CHARSET utf8
) RETURNS INT(11) # -1 or id
NO SQL
BEGIN
  DECLARE EXIT HANDLER FOR SQLEXCEPTION RETURN -1;

  INSERT INTO job_positions (Name) VALUES (pJobPositionName);
  RETURN LAST_INSERT_ID();
END//

DELIMITER ;

COMMIT ;
